@@ -52,12 +52,17 @@ class Client(object):
             "data": data
         })
 
+        self.manager = multiprocessing.Manager()
+        self.lock = multiprocessing.RLock()
         self.processes = process.Processes(self.create_socket)
-        self.create_safe()
-        self.create_options()
-        self.handle_signals()
+        self.commands = process.Commands(self.manager)
+
+        self.private = self.create_private()
+        self.public = self.create_public()
 
         self.standard_directory = os.path.join(duxlot.path, "standard")
+        self.populate_options()
+        self.handle_signals()
 
         self.load()
         self.setup()
@@ -72,33 +77,68 @@ class Client(object):
             debug("See http://stackoverflow.com/questions/2009278 to fix")
             sys.exit(1)
 
-    def create_safe(self):
-        self.safe = safe = duxlot.Storage()
+    def create_private(self):
+        private = duxlot.Storage()
 
-        safe.manager = multiprocessing.Manager()
-        safe.lock = multiprocessing.RLock()
+        private.command = self.commands.spawn
+        # private.events is set later on
+        # private.named is set later on
+        private.queue = {
+            "main": self.processes.queue,
+            "send": self.processes["send"].queue,
+            "messages": self.processes["messages"].queue,
+            "events": self.processes["events"].queue,
+            "schedule": self.processes["schedule"].queue
+        }
+        # private.socket is set in process.Process
 
-        safe.tasks = self.processes.queue
-        safe.sending = self.processes["send"].queue
-        safe.messages = self.processes["messages"].queue
-        safe.event_messages = self.processes["events"].queue
-        safe.schedule = self.processes["schedule"].queue
+        return private
 
-        safe.data = safe.manager.dict()
-        safe.options = options.Options(self.config.name, safe)
-        safe.commands = process.Commands(safe.manager, safe.schedule.put)
+    def create_public(self):
+        public = duxlot.Storage()
 
-        db_base = duxlot.config.path(self.config.base + ".database")
-        safe.database = duxlot.database(db_base, safe.manager.Namespace())
-        safe.send_message = create_send_message(safe.sending, debug)
+        public.data = self.manager.dict()
+        public.database = duxlot.database(
+            duxlot.config.path(self.config.base + ".database"),
+            self.manager.Namespace()
+        )
+        public.debug = debug
+        public.options = options.Options(
+            self.config.name,
+            self.manager,
+            public
+        )
 
-        def log(*text):
-            safe.tasks.put(("log",) + text)
-        safe.log = log
+        def schedule(*args):
+            self.private.queue["schedule"].put(args)
+        public.schedule = schedule
 
-    def create_options(self):
-        group = self.safe.options.group
-        option = self.safe.options.option
+        def task(*args):
+            self.private.queue["schedule"].put((0,) + args)
+        public.task = task
+
+        def send(*args):
+            if not all (isinstance(arg, str) for arg in args):
+                debug("Error: public.send:", args)
+                return
+    
+            if len(args) > 1:
+                args = list(args)
+                args[-1] = ":" + args[-1]
+    
+            octets = " ".join(args).encode("utf-8", "replace")
+            self.processes["send"].queue.put(octets)
+        public.send = send
+
+        def msg(*args):
+            send("PRIVMSG", *args)
+        public.msg = msg
+
+        return public
+
+    def populate_options(self):
+        group = self.public.options.group
+        option = self.public.options.option
         
         @group()
         class address(option):
@@ -140,7 +180,19 @@ class Client(object):
         
             # @@ reactions could be in the code setting the option...
             def react(self):
-                self.safe.send_message("NICK", self.data.value)
+                self.public.send("NICK", self.data.value)
+        
+        @group()
+        class modules(option):
+            "Modules to load"
+            default = [self.standard_directory]
+            types = {list}
+        
+            def parse(self, value):
+                ...
+        
+            def react(self):
+                self.public.task("reload")
         
         @group()
         class prefix(option):
@@ -162,19 +214,7 @@ class Client(object):
                     check(prefix)
                 return {"channels": value}
         
-        @group()
-        class standard(option):
-            "Modules to load"
-            default = []
-            types = {list}
-        
-            def parse(self, value):
-                ...
-        
-            def react(self):
-                self.safe.schedule.put((0, "reload"))
-        
-        self.safe.options.complete()
+        self.public.options.complete()
 
     def handle_signals(self):
         # http://stackoverflow.com/questions/2549939
@@ -192,13 +232,13 @@ class Client(object):
             user = os.environ.get("USER", "a signal")
             signame = signames.get(signum, signum)
             message = "%s made me do it (%s)" % (user, signame)
-            self.safe.send_message("QUIT", message)
+            self.public.send("QUIT", message)
 
             # This calls .finish(), so the QUIT above should work
             self.processes.stop()
 
             # Stop any stray commands, potentially corrupting queues
-            try: self.safe.commands.collect(0)
+            try: self.commands.collect(0)
             except: ...
 
             # Send SIGKILL to any remaining processes that we know of
@@ -219,41 +259,60 @@ class Client(object):
     def load(self):
         import importlib
 
+        def info(identifier):
+            if os.path.isdir(identifier):
+                for name in modules_in_directory(identifier):
+                    yield identifier, name
+            else:
+                yield os.path.split(identifier)
+
+        include = []
+        exclude = set()
+        standard = {"admin", "core", "services", "start", "text"}
+
+        identifiers = self.public.options("modules")
+        for identifier in identifiers:
+            if identifier.startswith("-"):
+                if os.path.isdir(identifier):
+                    continue # @@ Could remove this restriction
+                for directory, name in info(identifier):
+                    exclude.add((directory, name))
+            else:
+                for directory, name in info(identifier):
+                    include.append((directory, name))
+
         self.modules = {}
 
-        # @@ standard = set(self.safe.options("standard"))
-        directories = [self.standard_directory]
-        directories += self.safe.options("standard")
+        for directory, name in include:
+            if name in sys.modules:
+                debug("Warning: Skipping duplicate: %s" % module_name)
+                continue
 
-        for directory in directories:
-            module_names = modules_in_directory(directory)
+            if (directory, name) in exclude:
+                continue
 
-            # if directory == self.standard_directory:
-            #     if standard != {"*"}:
-            #         module_names = standard & set(module_names)
+            if directory == self.standard_directory:
+                if not (name in standard):
+                    # @@ User error message
+                    raise ValueError(name, "is not a standard module")
 
-            imported_names = []
-            for module_name in module_names:
-                if module_name in sys.modules:
-                    debug("Warning: Skipping duplicate: %s" % module_name)
-                    continue
-                identifier = (module_name, directory)
+            sys.path[:0] = [directory, duxlot.path]
+            module = importlib.import_module(name)
+            self.modules[(directory, name)] = module
+            sys.path[:2] = []
 
-                sys.path[:0] = [directory, duxlot.path]
-                self.modules[identifier] = importlib.import_module(module_name)
-                imported_names.append(module_name)
-                sys.path[:2] = []
-
-            # @@ reduce PWD too?
-            udir = duxlot.config.reduceuser(directory)
-            debug("Imported", ", ".join(imported_names), "from", udir)
+            if directory == self.standard_directory:
+                udir = "standard"
+            else:
+                udir = duxlot.config.reduceuser(directory)
+            debug(name, "imported from", udir)
 
     def reload(self, sender=None, nick=None):
         import imp
 
         def error(msg):
             if sender and nick:
-                self.safe.send_message("PRIVMSG", sender, nick + ": " + msg)
+                self.public.msg(sender, nick + ": " + msg)
 
         duxlot.clear()
 
@@ -261,7 +320,7 @@ class Client(object):
         imp.reload(api)
         sys.path[:1] = []
 
-        for (module_name, directory), module in self.modules.items():
+        for (directory, module_name), module in self.modules.items():
             sys.path[:0] = [directory, duxlot.path]
 
             try: reloaded = imp.reload(module)
@@ -272,7 +331,7 @@ class Client(object):
 
                 return False
             else:
-                self.modules[(module_name, directory)] = reloaded
+                self.modules[(directory, module_name)] = reloaded
             finally:
                 sys.path[:2] = []
 
@@ -281,26 +340,30 @@ class Client(object):
     def setup(self, react=False):
         # @@ Recording sources of commands
         for startup in duxlot.startups:
-            startup(self.safe)
+            startup(self.public)
 
-        self.safe.named = duxlot.commands.copy()
-        self.safe.events = duxlot.events.copy()
+        self.private.named = duxlot.commands.copy()
+        self.private.events = duxlot.events.copy()
 
-        self.safe.options.load(react=react)
+        self.public.options.load(react=react)
 
     def start(self):
-        self.processes["receive"].action(process_receive, self.safe)
-        self.processes["send"].action(process_send, self.safe)
-        self.processes["messages"].action(process_messages, self.safe)
-        self.processes["events"].action(process_events, self.safe)
-        self.processes["schedule"].action(process_schedule, self.safe)
+        functions = {
+            "receive": process_receive,
+            "send": process_send,
+            "messages": process_messages,
+            "events": process_events,
+            "schedule": process_schedule
+        }
 
+        for name, function in functions.items():
+            self.processes[name].action(function, self.private, self.public)
         self.processes.start()
 
     def create_socket(self):
         sock = socket.socket(socket.AF_INET, socket.TCP_NODELAY)
 
-        if self.safe.options("address", "ssl") is True:
+        if self.public.options("address", "ssl") is True:
             import ssl
             debug("Warning: Using SSL, but not validating the cert!")
             sock = ssl.wrap_socket(
@@ -310,8 +373,8 @@ class Client(object):
             )
 
         address = (
-            self.safe.options("address", "host"),
-            self.safe.options("address", "port")
+            self.public.options("address", "host"),
+            self.public.options("address", "port")
         )
 
         debug("Connecting to %s:%s" % address)
@@ -332,7 +395,7 @@ class Client(object):
             setattr(self.main, name, method)
 
         while True:
-            parameters = self.safe.tasks.get()
+            parameters = self.private.queue["main"].get()
             name, arguments = parameters[0], parameters[1:]
             debug("Got task:", name, arguments)
 
@@ -352,15 +415,19 @@ class Client(object):
         self.processes["events"].stop(finish=True)
         self.processes["messages"].stop(finish=True)
 
-        with self.safe.lock:
+        with self.lock:
             debug("Reloading...")
             success = self.reload(sender, nick)
             if success:
                 self.setup(react=True)
             debug("Reloaded")
 
-        self.processes["messages"].action(process_messages, self.safe)
-        self.processes["events"].action(process_events, self.safe)
+        self.processes["messages"].action(
+            process_messages, self.private, self.public
+        )
+        self.processes["events"].action(
+            process_events, self.private, self.public
+        )
 
         self.processes["messages"].start()
         self.processes["events"].start()
@@ -373,7 +440,7 @@ class Client(object):
             else:
                 msg = "The reload failed. Took %s seconds"
             msg = msg % round(duration, 3)
-            self.safe.send_message("PRIVMSG", sender, nick + ": " + msg)
+            self.public.msg(sender, nick + ": " + msg)
 
     @task
     def main_restart(self):
@@ -383,20 +450,19 @@ class Client(object):
         debug(" ")
 
         self.processes.empty()
-        if not self.safe.options("flood"):
+        if not self.public.options("flood"):
             time.sleep(3)
 
         elapsed = 0
-        while self.safe.commands.collectable(0):
+        while self.commands.collectable(0):
             time.sleep(1)
             elapsed += 1
             if elapsed > 6:
-                collected = self.safe.commands.collect(0)
+                # commands.collect waits for the processes to exit
+                collected = self.commands.collect(0)
                 debug("Restart collected", collected, "commands")
 
-                # Wait for the processes to exit
-                # Otherwise, they might add to the queues after .empty()
-                time.sleep(3)
+                # Should be safe to do this without delay
                 self.processes.empty()
 
         debug("Starting processes...")
@@ -410,14 +476,18 @@ class Client(object):
         debug("Disconnected")
         debug("")
 
-        if not self.safe.options("flood"):
+        if not self.public.options("flood"):
             time.sleep(6)
 
         self.processes.empty()
 
         debug("Reconnecting...")
-        self.processes["receive"].action(process_receive, self.safe)
-        self.processes["send"].action(process_send, self.safe)
+        self.processes["receive"].action(
+            process_receive, self.private, self.public
+        )
+        self.processes["send"].action(
+            process_send, self.private, self.public
+        )
 
         self.processes["receive"].start()
         self.processes["send"].start()
@@ -425,9 +495,9 @@ class Client(object):
     @task
     def main_quit(self, nick=None):
         if nick is not None:
-            self.safe.send_message("QUIT", "%s made me do it" % nick)
+            self.public.send("QUIT", "%s made me do it" % nick)
         else:
-            self.safe.send_message("QUIT")
+            self.public.send("QUIT")
 
         self.processes.stop()
         sys.exit(0)
@@ -443,32 +513,49 @@ class Client(object):
             number = len(process_names)
             names = ", ".join(sorted(process_names))
             msg = "%s processes are running (%s)" % (number, names)
-            self.safe.send_message("PRIVMSG", sender, nick + ": " + msg)
+            self.public.msg(sender, nick + ": " + msg)
 
-    # @@ main_pid
+    @task
+    def main_pids(self, sender, nick):
+        if sender and nick:
+            pids = ["receive: %s" % self.processes["receive"].process.pid]
+            for name in self.processes.queues:
+                pid = self.processes[name].process.pid
+                pids.append("%s: %s" % (name, pid))
+            text = ", ".join(pids)
+            self.public.msg(sender, nick + ": " + text)
 
     @task
     def main_collect(self, timeout=60):
-        collectable = self.safe.commands.collectable(timeout)
+        collectable = self.commands.collectable(timeout)
 
         if collectable:
             debug("Pausing processes")
+            # @@ Finish sending, join the sending queue
+            self.private.queue["send"].join()
+            self.processes.empty()
             self.processes.pause()
 
             debug("Collecting commands")
-            collected = self.safe.commands.collect(max(0, timeout - 10))
+            collected = self.commands.collect(max(0, timeout - 10))
             debug("Collected", collected, "commands")
 
             debug("Flushing queues")
             # self.processes.flush()
 
-            debug("Setting new queues")
-            self.safe.sending = self.processes["send"].queue
-            self.safe.schedule = self.processes["schedule"].queue
+            # @@ But we don't reset them!
+            # debug("Setting new queues")
+            # self.processes["send"].queue = multiprocessing.JoinableQueue()
+            # self.processes["schedule"].queue = multiprocessing.JoinableQueue()
+            # self.p... oh
 
             debug("Setting new actions")
-            self.processes["send"].action(process_send, self.safe)
-            self.processes["schedule"].action(process_schedule, self.safe)
+            self.processes["send"].action(
+                process_send, self.private, self.public
+            )
+            self.processes["schedule"].action(
+                process_schedule, self.private, self.public
+            )
 
             debug("Resuming...")
             self.processes.resume()
@@ -476,62 +563,65 @@ class Client(object):
 
     @task
     def main_ping(self):
-        self.safe.send_message("PING", self.safe.options("nick"))
+        self.public.send("PING", self.public.options("nick"))
 
     @task
     def main_ponged(self, pinged):
-        ponged = self.safe.data.get("ponged", 0)
+        ponged = self.public.data.get("ponged", 0)
         if ponged < pinged:
             self.main.restart()
 
     @task
     def main_msg(self, recipient, text):
-        self.safe.send_message("PRIVMSG", recipient, text)
+        self.public.msg(recipient, text)
 
 ### Processes ###
 
 # (a) process:receive
-def process_receive(safe):
+def process_receive(private, public):
     debug("START! process_receive")
     import ssl
 
-    def receive_loop(safe, sockfile):
+    def receive_loop(sockfile, private):
         count = 0
         for octets in sockfile:
             o = api.irc.parse_message(octets=octets)
             count += 1
             o.count = count
-            safe.messages.put(o())
+            private.queue["messages"].put(o())
 
             # @@ debug here can hang if there are pipe problems
             debug("RECV:", octets)
 
-    with safe.socket.makefile("rb") as sockfile:
-        try: receive_loop(safe, sockfile)
+    with private.socket.makefile("rb") as sockfile:
+        try: receive_loop(sockfile, private)
         except (IOError, EOFError, socket.error, ssl.SSLError):
             # @@ debug here can hang if there are pipe problems
             debug("Got socket or SSL error")
-            safe.tasks.put(("restart",))
+            public.task("restart")
         else:
             # @@ debug here can hang if there are pipe problems
             debug("Got regular disco")
-            safe.tasks.put(("restart",)) # @@
+            public.task("restart") # @@
 
     # @@ debug here can hang if there are pipe problems
     debug("DONE! process:receive")
 
 # (b) process:send
-def process_send(safe):
+def process_send(private, public):
     import ssl
 
     debug("START! process_send")
 
+    send_get = private.queue["send"].get
+    send_done = private.queue["send"].task_done
+
     def send_loop():
         sent = 0
-
-        with safe.socket.makefile("wb") as sockfile:
+        
+        with private.socket.makefile("wb") as sockfile:
             while True:
-                octets = safe.sending.get()
+                octets = send_get()
                 if octets == "StopIteration":
                     break
     
@@ -544,7 +634,7 @@ def process_send(safe):
                 # @@ If we wait for QUIT, the socket can I/O error
                 # This is strange because the receive socket is fine
                 if not octets.startswith(b"QUIT"):
-                    if not safe.options("flood"):
+                    if not public.options("flood"):
                         now = time.time()
                         if sent > (now - 1):
                             time.sleep(0.5)
@@ -555,42 +645,78 @@ def process_send(safe):
 
                 # @@ debug here can hang if there are pipe problems
                 debug("SENT:", octets + b"\r\n")
-                safe.sending.task_done()
+                send_done()
 
     try: send_loop()
     except (IOError, EOFError, socket.error, ssl.SSLError) as err:
         debug("Send Error:", err.__class__.__name__, err)
-        ...
 
-    safe.sending.task_done()
+    send_done()
     # @@ debug here can hang if there are pipe problems
     debug("DONE! process:send")
 
 # (c) process:messages
-def process_messages(safe):
+def process_messages(private, public):
     debug("START! process_messages")
-    safe.database.cache.usage = safe.database.load("usage") or {}
+    public.database.cache.usage = public.database.load("usage") or {}
+    messages_get = private.queue["messages"].get
     while True:
         # debug("Waiting for message")
-        message = safe.messages.get()
+        message = messages_get()
         # debug("Got message", message)
         if message == "StopIteration":
             break
 
         if message["command"] == "PRIVMSG":
-            handle_named(safe, safe.named, message)
-        safe.event_messages.put(message)
-        safe.messages.task_done()
+            env = create_irc_env(public, message)
+        
+            if "command" in env:
+                if env.command in private.named:
+                    def process_command(env):
+                        # @@ pre-command
+                        try: private.named[env.command](env)
+                        except api.Error as err:
+                            env.say("Error: %s" % err)
+                        except Exception as err:
+                            import os.path
+                            import traceback
+        
+                            name = err.__class__.__name__
+                            tb = err.__traceback__
+                            stack = traceback.extract_tb(tb, limit=2)
+                            item = list(stack.pop())
+                            item[0] = os.path.basename(item[0])
+                            where = "%s:%s at %s(...) %s" % tuple(item)
+                            msg = "Script Error: %s: %s, in %s"
+                            env.say(msg % (name, err, where))
+        
+                            debug("---")
+                            for line in traceback.format_exception(
+                                    err.__class__, err, err.__traceback__):
+                                line = line.rstrip("\n")
+                                line = line.replace(duxlot.path + os.sep, "")
+                                debug(line)
+                            debug("---")
+                        # @@ post-command
+        
+                        with public.database.context("usage") as usage:
+                            usage.setdefault(env.command, 0)
+                            usage[env.command] += 1
+        
+                    private.command(process_command, env)
 
-    safe.messages.task_done()
+        private.queue["events"].put(message)
+        private.queue["messages"].task_done()
+
+    private.queue["messages"].task_done()
     # @@ debug here can hang if there are pipe problems
     debug("DONE! process:messages")
 
 # (d) process:events
-def process_events(safe):
+def process_events(private, public):
     debug("START! process_events")
     while True:
-        message = safe.event_messages.get()
+        message = private.queue["events"].get()
         if message == "StopIteration":
             break
 
@@ -598,10 +724,10 @@ def process_events(safe):
         if message["count"] == 1:
             commands = ["1st"] + commands
 
-        env = create_irc_env(safe, message)
+        env = create_irc_env(public, message)
         for priority in ["high", "medium", "low"]:
             for command in commands:
-                for function in safe.events[priority].get(command, []):
+                for function in private.events[priority].get(command, []):
                     def process_command(env):
                         try: function(env)
                         except Exception as err:
@@ -610,27 +736,27 @@ def process_events(safe):
                     if not hasattr(function, "concurrent"):
                         process_command(env)
                     elif function.concurrent:
-                        safe.commands.spawn(process_command, env)
+                        private.command(process_command, env)
                     else:
                         process_command(env)
 
-        safe.event_messages.task_done()
-    safe.event_messages.task_done()
+        private.queue["events"].task_done()
+    private.queue["events"].task_done()
     # @@ debug here can hang if there are pipe problems
     debug("DONE! process:events")
 
 # (e) process:schedule
-def process_schedule(safe):
+def process_schedule(private, public):
     debug("START! process_schedule")
 
     import heapq
     import queue
     import time
 
-    database = safe.database
-    # @@ safe.queue is not reliable!
-    receive = safe.schedule.get
-    task = safe.tasks.put
+    database = public.database
+    # @@ the set queue is not reliable!
+    receive = private.queue["schedule"].get
+    task = private.queue["main"].put
 
     duration = 1/3
     # @@ init won't work here, doesn't return anything
@@ -654,7 +780,7 @@ def process_schedule(safe):
     periodic.called = {}
     periodic.stamp = {}
 
-    # @periodic(safe.opt.time_between_pings_in_seconds)
+    # @periodic(public.options("time_between_pings_in_seconds"))
     @periodic(300)
     def ping(current):
         task(("ping",))
@@ -766,65 +892,25 @@ def process_schedule(safe):
 
 ### Process anciliaries ###
 
-# used by process:messages
-def handle_named(safe, named, message):
-    env = create_irc_env(safe, message)
-
-    if "command" in env:
-        if env.command in named:
-            def used(cmd):
-                with safe.database.context("usage") as usage:
-                    usage.setdefault(cmd, 0)
-                    usage[cmd] += 1
-
-            def process_command(env):
-                # @@ multiprocessing.managers.RemoteError
-                # env = create_irc_env(safe, message) # @@!
-                # @@ pre-command
-                try: named[env.command](env)
-                except api.Error as err:
-                    env.say("Error: %s" % err)
-                except Exception as err:
-                    import os.path
-                    import traceback
-
-                    name = err.__class__.__name__
-                    stack = traceback.extract_tb(err.__traceback__, limit=2)
-                    item = list(stack.pop())
-                    item[0] = os.path.basename(item[0])
-                    where = "%s:%s at %s(...) %s" % tuple(item)
-                    env.say("Script Error: %s: %s, in %s" % (name, err, where))
-
-                    debug("---")
-                    for line in traceback.format_exception(
-                            err.__class__, err, err.__traceback__):
-                        line = line.rstrip("\n")
-                        line = line.replace(duxlot.path + os.sep, "")
-                        debug(line)
-                    debug("---")
-                # @@ post-command
-                used(env.command)
-
-            safe.commands.spawn(process_command, env)
-    debug("Quitting handle_named")
-
-def create_irc_env(safe, message):
-    env = duxlot.Storage()
+def create_irc_env(public, message):
+    proto = public()
+    env = duxlot.Storage(proto.copy())
 
     env.message = message
     env.event = message["command"]
+
     if "prefix" in message:
         env.nick = message["prefix"]["nick"]
 
     if env.event == "PRIVMSG":
         env.sender = message["parameters"][0]
         env.text = message["parameters"][1]
-        if "address" in safe.data:
-            env.limit = 498 - len(env.sender) + len(safe.data["address"])
+        if "address" in public.data:
+            env.limit = 498 - len(env.sender + public.data["address"])
 
-        prefix = safe.options("prefix", "channels").get(env.sender)
+        prefix = public.options("prefix", "channels").get(env.sender)
         if prefix is None:
-            prefix = safe.options("prefix", "channels").get("")
+            prefix = public.options("prefix", "channels").get("", ".") # @@
         env.prefix = prefix
 
         if env.text.startswith(prefix):            
@@ -835,72 +921,32 @@ def create_irc_env(safe, message):
             else:
                 env.command, env.arg = sans_prefix, ""
 
-        env.private = env.sender == safe.options("nick")
+        env.private = env.sender == public.options("nick")
         if env.private:
             env.sender = env.nick
 
-    env.data = safe.data
-    env.options = safe.options
-    env.lock = safe.lock
-    # @@ schedule wrapper?
-    # env.task = safe.tasks.put
-    env.database = safe.database
-    # env.schedule = safe.schedule.put
-    env.sent = safe.sending.join
-    env.log = safe.log
-
     if "sender" in env:
         def say(text):
-            safe.send_message("PRIVMSG", env.sender, text)
+            public.msg(env.sender, text)
         env.say = say
 
     if ("nick" in env) and ("sender" in env):
         def reply(text):
-            text = env.nick + ": " + text
-            safe.send_message("PRIVMSG", env.sender, text)
+            public.msg(env.sender, env.nick + ": " + text)
         env.reply = reply
 
-    # @@ functionise
-    env.task = lambda *args: safe.schedule.put((0,) + args)
-    env.schedule = lambda *args: safe.schedule.put(args)
-
-    def send(*arguments):
-        safe.send_message(*arguments)
-    env.send = send
-
-    def msg(recipient, text):
-        safe.send_message("PRIVMSG", recipient, text)
-    env.msg = msg
-
     # @@ this shouldn't really be here...
     for builder in duxlot.builders:
         env = builder(env)
 
     return env
 
-# process-safe
-def create_send_message(sending, log):
-    def send_message(*arguments):
-        arguments = list(arguments)
-
-        if not all (isinstance(arg, str) for arg in arguments):
-            log("Error: send_message: %s" % arguments)
-            return
-
-        if len(arguments) > 1: # and " " in arguments[-1]
-            arguments[-1] = ":" + arguments[-1]
-
-        sending.put(" ".join(arguments).encode("utf-8", "replace"))
-    return send_message
-
 # @@ a more efficient message parser
 # @@ a more efficient environment builder
 # @@ a naïve timer for other commands
 # @@ separate namespace for command chaining state
-# @@ note where the lock is used, and the types of usage
 # @@ ..commands for web services, or .regular and .services other
 # @@ last n seconds of events, event list
-# @@ reload config. nick/channels/prefix might change, etc.
 # @@ make F008 work, and the character itself
 # @@ make it clear when a child process goes boom
 # @@ ignore list
@@ -916,7 +962,5 @@ def create_send_message(sending, log):
 # @@ relative Location bug
 # @@ better config documentation
 # @@ credentialise admin.py
-# @@ bot / task and schedule functions for commands to use
-# @@ writebackable config
 # @@ refuse to run in .duxlot-src
 # @@ reply should automatically not prepend nick in private
